module MyGrammar where

import Grammar

myGrammar :: Grammar
myGrammar = Grammar {
	mainRule = "statement",

	skipped = [
		" ",
		"\t",
		"#.*"],

	delimiters = [
		".",
		",",
		";",
		"(",
		")"],
	
	rules = [
		("statement",      Syntax statement),

		("symbol-def",     Syntax symbolDef),
		("function-def",   Syntax functionDef),
		("lambda-def",     Syntax lambdaDef),

		("parameters",     Syntax parameters),
		("arguments",      Syntax arguments),
		
		("expression",     Syntax expression),
		("sub-expression", Syntax subExpression),
		("something",      Syntax something),

		("function-call",  Syntax functionCall),
		("atom",           Syntax atom),


		("operator", 	Token Bounded "[\\+-\\*/]"), --"~(-?[0-9]).*"
		("string", 	Token Greedy  "\".*\""),
		("char", 	Token Greedy  "('.')|('\\.')"),
		("decimal", 	Token Bounded "-?[0-9]+\\.[0-9]+"),
		("int", 	Token Bounded "-?[0-9]+"),
		("date",	Token Bounded "([0-9]{4}-[0-9]{2}-[0-9]{2})|([0-9]{4}/[0-9]{2}/[0-9]{2})"),
		("time",	Token Bounded "[0-9]{2}:[0-9]{2}:[0-9]{2}"),
		("symbol",	Token Bounded ".*") --"~(-?[0-9]).*"
		]
	}




statement
	= Either [
		Rule "symbol-def",
		Rule "function-def"]


symbolDef
	= Sequence [
		Rule "symbol",
		DirectToken "=",
		Rule "expression"]


functionDef
	= Sequence [
		DirectToken "function",
		Rule "symbol",
		DirectToken "(",
		Rule "parameters",
		DirectToken ")",
		DirectToken "=",
		Rule "expression"]


lambdaDef
	= Sequence [
		DirectToken "function",
		DirectToken "(",
		Rule "parameters",
		DirectToken ")",
		DirectToken "->",
		Rule "expression"]


functionCall
	= Sequence [
		Rule "symbol",
		DirectToken "(",
		Rule "arguments",
		DirectToken ")"]


parameters
	= Maybe ( Sequence [
		Rule "symbol", 
		ZeroOrMore ( Sequence [
			DirectToken ",",
			Rule "symbol"
			])
		])


arguments
	= Maybe ( Sequence [
		Rule "expression", 
		ZeroOrMore ( Sequence [
			DirectToken ",",
			Rule "expression"
			])
		])


expression
	= Sequence [
		Rule "something",
		ZeroOrMore ( Sequence [
			Rule "operator",
			Rule "something"
			])
		]

subExpression
	= Sequence [
		DirectToken "(",
		Rule "expression",
		DirectToken ")"
		]

something
	= Either [
		Rule "sub-expression",
		Rule "lambda-def",
		Rule "function-call",
		Rule "atom"
		]


atom
	= Either [
		Rule "string",
		Rule "char",
		Rule "decimal",
		Rule "int",
		Rule "date",
		Rule "time",
		Rule "symbol"
		]

